Material resource
{
	MaterialType _type = Blended;
	VideoProgram _passes
	[ 
		"normal"
	]
}

VideoProgram normal
{
	VertexLayout _vertexLayout = "MeshLayout\TerrainLayout.rsc";
	VertexProgram _vertexProgram = "vs";
	PixelProgram _pixelProgram = "Material/River/RiverMaterial.rsc:ps";
	RenderState _renderState = "Material/River/RiverMaterial.rsc:rs";
}

VertexProgram vs
{
	VertexLayout _input = "MeshLayout/TerrainLayout.rsc";
	String _includes
	[
		"Material/Instancing.hlsl"
		"Material/Lighting.hlsl"
	]

	Output _outputs 
	[
		{ String _type = "float4"; String _name = "position";			Usage _usage = Position;			int _index = 0; }
		{ String _type = "float4"; String _name = "texcoord";			Usage _usage = TextureCoordinate;	int _index = 0; }
		{ String _type = "float3"; String _name = "viewDir";			Usage _usage = TextureCoordinate;	int _index = 1; }
		{ String _type = "float4"; String _name = "screenProjection";	Usage _usage = TextureCoordinate;	int _index = 2; }
		{ String _type = "float4"; String _name = "lightFogAlpha";		Usage _usage = TextureCoordinate;	int _index = 3; }
	]

	String _functions
	[
		"
			void Common(float4x4 localToWorld, float4 position, float3 normal, float2 texcoord, out float3 worldPosition, out float4 outputPosition,
				out float4 outputTexcoord, out float4 lightFogAlpha, out float4 screenProjection, out float3 viewDir)
			{
				position.z /= 256.0;
				position.xy /= 48.0;
				worldPosition = mul((float3x4)localToWorld, float4(position.xyz, 1.0));
				outputPosition = mul(gc_worldToProjection, float4(worldPosition, 1.0));

				float4x4 so = float4x4(
					float4(0.5, 0.0, 0.0, 0.5),
					float4(0.0, -0.5, 0.0, 0.5),
					float4(0.0, 0.0, 1.0, 0.0),
					float4(0.0, 0.0, 0.0, 1.0));
		
				screenProjection = mul(so, outputPosition);
	
				lightFogAlpha.xyz = GetLightingFog(float3(0, 0, 1), outputPosition.z);
				lightFogAlpha.w = normal.z;
	
				outputTexcoord.xy = texcoord.xy / 1024.0;
				outputTexcoord.zw = texcoord.xy / 1024.0;
				outputTexcoord.wz += gc_time * float2(0.01, 0.005);
				outputTexcoord.y += gc_time * -0.006 * float2(0.95, 1.05);
	
				viewDir = worldPosition - gc_cameraPosition.xyz;
			}
		"
	]

	Variant _variants
	[
		{
			String _text = "
				float4x4 localToWorld = GetLocalToWorld(0);
				float3 worldPosition;
				Common(localToWorld, input.position, input.normal.xyz, input.texcoord, worldPosition, output.position, output.texcoord, output.lightFogAlpha, output.screenProjection, output.viewDir);
			";
		}
		{
			Output _outputs 
			[
				{ String _type = "float4"; String _name = "shadowProjection";	Usage _usage = TextureCoordinate;	int _index = 4; }
			]
			String _text = "
				float4x4 localToWorld = GetLocalToWorld(0);
				float3 worldPosition;
				Common(localToWorld, input.position, input.normal.xyz, input.texcoord, worldPosition, output.position, output.texcoord, output.lightFogAlpha, output.screenProjection, output.viewDir);
				output.shadowProjection = mul(lc_shadowProjection0, float4(worldPosition, 1.0f));
			";
		}
		{
			Output _outputs 
			[
				{ String _type = "float4"; String _name = "shadowProjection";	Usage _usage = TextureCoordinate;	int _index = 4; }
			]
			String _text = "
				float4x4 localToWorld = GetLocalToWorld(0);
				float3 worldPosition;
				Common(localToWorld, input.position, input.normal.xyz, input.texcoord, worldPosition, output.position, output.texcoord, output.lightFogAlpha, output.screenProjection, output.viewDir);
				output.shadowProjection = mul(lc_shadowProjection0, float4(worldPosition, 1.0f));
			";
		}
		{
			Output _outputs 
			[
				{ String _type = "float4"; String _name = "shadowProjection";	Usage _usage = TextureCoordinate;	int _index = 4; }
			]
			String _text = "
				float4x4 localToWorld = GetLocalToWorld(0);
				float3 worldPosition;
				Common(localToWorld, input.position, input.normal.xyz, input.texcoord, worldPosition, output.position, output.texcoord, output.lightFogAlpha, output.screenProjection, output.viewDir);
				output.shadowProjection = mul(lc_shadowProjection0, float4(worldPosition, 1.0f));
			";
		}
	]
}



PixelProgram ps
{
	VertexProgram _input = "vs";
	String _includes
	[
		"Material/Texture.hlsl"
		"Material/Shadows.hlsl"
	]

	Texture _textureList
	[
		{ bool _2d = true;	String _name = "water";			int _bindIndex = 0;	bool _exposed = true; }
		{ bool _2d = true;	String _name = "frame";			int _bindIndex = 1;	bool _exposed = true; }
		{ bool _2d = false;	String _name = "reflection";	int _bindIndex = 2;	bool _exposed = true; }
		{ bool _2d = true;	String _name = "shadowMap";		int _bindIndex = 5;	bool _exposed = false; bool _shadowMap = true; }
	]

	String _functions
	[
		"
		float4 GetColor(float4 texcoord, float4 screenProjection, float3 lightFog, float3 viewDir, float shadow)
		{
			float3 normalA, normalB; 
			normalA.xy = Sample2D(water, watersampler, texcoord.xy).wy * 2 - 1;
			normalA.xy += Sample2D(water, watersampler, texcoord.zw).wy * 2 - 1;
			normalA.xy *= 0.5;
			normalA.z = 1.0 - sqrt(dot(normalA.xy, normalA.xy));

			float3 normal = normalA;
			screenProjection.xy += (normal.xy) * 1;
			float4 frameBuffer = Sample2DProj(frame, framesampler, screenProjection);
    
			float d = dot(-normalize(viewDir), normal);
			
    
			float3 dir = reflect(normalize(viewDir), normal);
			float3 ref = SampleCUBE(reflection, reflectionsampler, dir).xyz * 0.66;
			
			d = d * d * d * d;
			d = 1 - ((1 - d) * (frameBuffer.a));

			float3 diffuse = lerp(frameBuffer.xyz, float3(0.0, 0.0, 0.02), frameBuffer.a * saturate(length(viewDir) * 0.01));
			diffuse = lerp(ref, diffuse, d);
            
			float4 output;
			output.xyz = diffuse;// * (shadow * 0.5 + 0.5);
			output.xyz = lerp(output.xyz, pc_fogColor.xyz, saturate(lightFog.z) * frameBuffer.a);
			output.w = 0.0;

			return output;
		}
		"
	]

	Variant _variants
	[
		{
			String _text = "output.color = GetColor(input.texcoord, input.screenProjection, input.lightFog, input.viewDir, 1);";
		}
		{
			String _text = "output.color = GetColor(input.texcoord, input.screenProjection, input.lightFog, input.viewDir, GetShadowLow(input.shadowProjection));";
		}
		{
			String _text = "output.color = GetColor(input.texcoord, input.screenProjection, input.lightFog, input.viewDir, GetShadowMedium(input.shadowProjection, pc_texelSize.x));";
		}
		{
			String _text = "output.color = GetColor(input.texcoord, input.screenProjection, input.lightFog, input.viewDir, GetShadowHigh(input.shadowProjection, pc_texelSize.x));";
		}
	]
}