Material resource
{
	MaterialType _type = Opaque;
	VideoProgram _passes
	[ 
		"normal"
		"depth"
		"reflect"
	]
}

VideoProgram normal
{
	VertexLayout _vertexLayout = "MeshLayout\ModelLayout.rsc";
	VertexProgram _vertexProgram = "vs";
	PixelProgram _pixelProgram = "ps";
	RenderState _renderState = "rs";
}

VideoProgram depth
{
	VertexLayout _vertexLayout = "MeshLayout\ModelDepthLayout.rsc";
	VertexProgram _vertexProgram = "vsDepth";
	PixelProgram _pixelProgram = "psDepth";
	RenderState _renderState = "rsDepth";
}

VideoProgram reflect : "normal"
{
	RenderState _renderState = "rsRef";
}

RenderState rs
{
	AlphaBlend _alphaBlend
	{
		bool _enable = false;
		Blend _source = One;
		Blend _dest = Zero;
		BlendOp _op = Add;
		ColorWrite _writeMask = Red | Green | Blue | Alpha;
	}
	Depth _depth
	{
		bool _enable = true;
		bool _write = true;
		Compare _compare = LessEqual;
		float _bias = 0.000000;
		float _slopeBias = 0.000000;
	}
	Stencil _stencil
	{
		bool _enable = false;
		bool _twosided = false;
		Compare _compare = Greater;
		uchar _ref = 0;
		StencilOp _fail = Keep;
		StencilOp _zFail = Keep;
		StencilOp _pass = Keep;
		StencilOp _tsFail = Keep;
		StencilOp _tsZFail = Keep;
		StencilOp _tsPass = Keep;
	}
	CullMode _cullMode = BackFaces;
	CullMode _fillMode = Solid;
}

RenderState rsRef : "rs"
{
	CullMode _cullMode = FrontFaces;
}

RenderState rsRefNone : "rs"
{
	CullMode _cullMode = None;
}

RenderState rsDepth
{
	AlphaBlend _alphaBlend
	{
		bool _enable = false;
		Blend _source = One;
		Blend _dest = Zero;
		BlendOp _op = Add;
		ColorWrite _writeMask = 0;
	}
	Depth _depth
	{
		bool _enable = true;
		bool _write = true;
		Compare _compare = LessEqual;
		float _bias = 0.0;//0.0010;
		float _slopeBias = 0.0;//0.000100;
	}
	Stencil _stencil
	{
		bool _enable = false;
		bool _twosided = false;
		Compare _compare = Greater;
		uchar _ref = 0;
		StencilOp _fail = Keep;
		StencilOp _zFail = Keep;
		StencilOp _pass = Keep;
		StencilOp _tsFail = Keep;
		StencilOp _tsZFail = Keep;
		StencilOp _tsPass = Keep;
	}
	CullMode _cullMode = FrontFaces;
	CullMode _fillMode = Solid;
}

VertexProgram vs
{
	bool _instanced = true;
	VertexLayout _input = "MeshLayout/ModelLayout.rsc";
	String _includes
	[
		"Material/Instancing.hlsl"
		"Material/Lighting.hlsl"
	]

	Output _outputs 
	[
		{ String _type = "float4"; String _name = "position";		Usage _usage = Position;			int _index = 0; }
		{ String _type = "float4"; String _name = "texcoord";		Usage _usage = TextureCoordinate;	int _index = 0; }
		{ String _type = "float3"; String _name = "lightFog";		Usage _usage = TextureCoordinate;	int _index = 1; }
	]

	String _functions
	[
		"
			void Common(float4x4 localToWorld, float4 position, float4 texcoord, float3 normal, out float3 worldPosition, out float4 outputPosition,
						out float4 outputTexcoord, out float3 lightFog)
			{
				position.xyz *= 1.0 / 512.0;
				position.xyz *= localToWorld[3].xyz;
	
				worldPosition = mul((float3x4)localToWorld, float4(position.xyz, 1.0));
				outputPosition = mul(gc_worldToProjection, float4(worldPosition, 1.0));
	
				normal = normalize(normal * 2 - 1);
				normal = mul((float3x3)localToWorld, normal).xyz;
	
				lightFog = GetLightingFog(normal, outputPosition.z);
	
				outputTexcoord = texcoord / float4(512.0, 512.0, 8192.0, 8192.0);
			}
		"
	]

	Variant _variants
	[
		{
			String _text = "
				float4x4 localToWorld = GetLocalToWorld(input.position.w);
				float3 worldPosition;
				Common(localToWorld, input.position, input.texcoord, input.normal.xyz, worldPosition, output.position, output.texcoord, output.lightFog);
			";
		}
		{
			Output _outputs 
			[
				{ String _type = "float4"; String _name = "shadowProjection";	Usage _usage = TextureCoordinate;	int _index = 2; }
			]
			String _text = "
				float4x4 localToWorld = GetLocalToWorld(input.position.w);
				float3 worldPosition;
				Common(localToWorld, input.position, input.texcoord, input.normal.xyz, worldPosition, output.position, output.texcoord, output.lightFog);
				output.shadowProjection = mul(lc_shadowProjection0, float4(worldPosition, 1.0f));
			";
		}
		{
			Output _outputs 
			[
				{ String _type = "float4"; String _name = "shadowProjection";	Usage _usage = TextureCoordinate;	int _index = 2; }
			]
			String _text = "
				float4x4 localToWorld = GetLocalToWorld(input.position.w);
				float3 worldPosition;
				Common(localToWorld, input.position, input.texcoord, input.normal.xyz, worldPosition, output.position, output.texcoord, output.lightFog);
				output.shadowProjection = mul(lc_shadowProjection0, float4(worldPosition, 1.0f));
			";
		}
		{
			Output _outputs 
			[
				{ String _type = "float4"; String _name = "shadowProjection";	Usage _usage = TextureCoordinate;	int _index = 2; }
			]
			String _text = "
				float4x4 localToWorld = GetLocalToWorld(input.position.w);
				float3 worldPosition;
				Common(localToWorld, input.position, input.texcoord, input.normal.xyz, worldPosition, output.position, output.texcoord, output.lightFog);
				output.shadowProjection = mul(lc_shadowProjection0, float4(worldPosition, 1.0f));
			";
		}
	]
}

PixelProgram ps
{
	VertexProgram _input = "vs";
	String _includes
	[
		"Material/Texture.hlsl"
		"Material/Shadows.hlsl"
	]

	Texture _textureList
	[
		{ bool _2d = true;	String _name = "diffuse";	int _bindIndex = 0;	bool _exposed = true; }
		{ bool _2d = true;	String _name = "aoMap";		int _bindIndex = 3;	bool _exposed = false; }
		{ bool _2d = true;	String _name = "shadowMap";	int _bindIndex = 5;	bool _exposed = false; bool _shadowMap = true; }
	]

	String _functions
	[
		"
			float4 GetColor(float4 texcoord, float3 lightFog, float shadow)
			{
				float3 ao = Sample2D(aoMap, aoMapsampler, texcoord.zw).xxx;
				float4 color = Sample2D(diffuse, diffusesampler, texcoord.xy);
     
				float4 output; 
				output.xyz = ComputeLighting(lightFog, color.xyz, shadow, float3(1, 1, 1), ao);
				output.w = color.w;
	
				return output;
			}
		"
	]

	Variant _variants
	[
		{
			String _text = "output.color = GetColor(input.texcoord, input.lightFog, 1);";
		}
		{
			String _text = "output.color = GetColor(input.texcoord, input.lightFog, GetShadowLow(input.shadowProjection));";
		}
		{
			String _text = "output.color = GetColor(input.texcoord, input.lightFog, GetShadowMedium(input.shadowProjection, pc_texelSize.x));";
		}
		{
			String _text = "output.color = GetColor(input.texcoord, input.lightFog, GetShadowHigh(input.shadowProjection, pc_texelSize.x));";
		}
	]
}

VertexProgram vsDepth
{
	bool _instanced = true;
	VertexLayout _input = "MeshLayout/ModelDepthLayout.rsc";
	String _includes
	[
		"Material/Instancing.hlsl"
	]

	Output _outputs 
	[
		{ String _type = "float4"; String _name = "position";		Usage _usage = Position;			int _index = 0; }
	]

	Variant _variants
	[
		{
			String _text = "
				float4x4 localToWorld = GetLocalToWorld(input.position.w);
				float3 position = input.position.xyz * localToWorld[3].xyz / 512.0;
				float3 worldPosition = mul((float3x4)localToWorld, float4(position.xyz, 1.0));
	
				output.position = mul(gc_worldToProjection, float4(worldPosition, 1.0));
			";
		}
	]
}

PixelProgram psDepth
{
	VertexProgram _input = "vsDepth";

	Variant _variants
	[
		{
			String _text = "
				output.color = float4(1.0, 1.0, 1.0, 1.0);
			";
		}
	]
}