#ifndef _Shadows_
#define _Shadows_

float3 ComputeLighting(float3 lightingFog, float3 diffuse, float shadow, float3 lightColor, float3 ambientColor)
{
	// compute ambient and direct light
	float3 directLight = pc_lightColor.xyz * shadow * saturate(lightingFog.x) * lightColor;
	float3 ambientLight = pc_ambientColor.xyz * lightingFog.y * ambientColor;
	float3 lighting = (directLight + ambientLight) * diffuse;

	// apply fog
	return lerp(lighting, pc_fogColor.xyz, saturate(min(lightingFog.z, pc_fogColor.w)));
}

float3 ComputeLighting(float3 lightingFog, float3 diffuse, float shadow)
{
	// compute ambient and direct light
	float3 directLight = pc_lightColor.xyz * shadow * saturate(lightingFog.x);
	float3 ambientLight = pc_ambientColor.xyz * lightingFog.y;
	float3 lighting = (directLight + ambientLight) * diffuse;

	// apply fog
	return lerp(lighting, pc_fogColor.xyz, saturate(min(lightingFog.z, pc_fogColor.w)));
}

float GetShadowHigh(float4 projection, float texelSize)
{
	float4 offset = float4(texelSize, -texelSize, 0.0, 0.0) * projection.w;
	float4 offset2 = float4(2.0 * texelSize, -2.0 * texelSize, 0.0, 0.0) * projection.w;
	
    float shadow0 = SampleShadow(shadowMap, shadowMapsampler, projection + offset.xxzz).x;
    float shadow1 = SampleShadow(shadowMap, shadowMapsampler, projection + offset.xyzz).x;
    float shadow2 = SampleShadow(shadowMap, shadowMapsampler, projection + offset.yxzz).x;
    float shadow3 = SampleShadow(shadowMap, shadowMapsampler, projection + offset.yyzz).x;
    float shadow4 = SampleShadow(shadowMap, shadowMapsampler, projection).x;
    float shadow5 = SampleShadow(shadowMap, shadowMapsampler, projection + offset.xzzz).x;
    float shadow6 = SampleShadow(shadowMap, shadowMapsampler, projection + offset.yzzz).x;
    float shadow7 = SampleShadow(shadowMap, shadowMapsampler, projection + offset.zxzz).x;
    float shadow8 = SampleShadow(shadowMap, shadowMapsampler, projection + offset.zyzz).x;

    return (shadow0 + shadow1 + shadow2 + shadow3 + shadow4 + shadow5 + shadow6 + shadow7 + shadow8) * (1.0 / 9.0);
}

float GetShadowMedium(float4 projection, float texelSize)
{
	float4 offset = float4(texelSize * 0.5, -texelSize * 0.5, 0.0, 0.0) * projection.w;
	
    float shadow0 = SampleShadow(shadowMap, shadowMapsampler, projection + offset.xxzz).x;
    float shadow1 = SampleShadow(shadowMap, shadowMapsampler, projection + offset.xyzz).x;
    float shadow2 = SampleShadow(shadowMap, shadowMapsampler, projection + offset.yxzz).x;
    float shadow3 = SampleShadow(shadowMap, shadowMapsampler, projection + offset.yyzz).x;
   // float shadow4 = SampleShadow(shadowMap, shadowMapsampler, projection).x;

    return (shadow0 + shadow1 + shadow2 + shadow3) * (1.0 / 4.0);
}

float GetShadowHighStar(float4 projection, float texelSize)
{
	float4 offset = float4(texelSize, -texelSize, 0.0, 0.0) * projection.w;
	float4 offset2 = float4(2.0 * texelSize, -2.0 * texelSize, 0.0, 0.0) * projection.w;
	
    float shadow0 = SampleShadow(shadowMap, shadowMapsampler, projection + offset.xxzz).x;
    float shadow1 = SampleShadow(shadowMap, shadowMapsampler, projection + offset.xyzz).x;
    float shadow2 = SampleShadow(shadowMap, shadowMapsampler, projection + offset.yxzz).x;
    float shadow3 = SampleShadow(shadowMap, shadowMapsampler, projection + offset.yyzz).x;
    float shadow4 = SampleShadow(shadowMap, shadowMapsampler, projection).x;
    float shadow5 = SampleShadow(shadowMap, shadowMapsampler, projection + offset2.xzzz).x;
    float shadow6 = SampleShadow(shadowMap, shadowMapsampler, projection + offset2.yzzz).x;
    float shadow7 = SampleShadow(shadowMap, shadowMapsampler, projection + offset2.zxzz).x;
    float shadow8 = SampleShadow(shadowMap, shadowMapsampler, projection + offset2.zyzz).x;

    return (shadow0 + shadow1 + shadow2 + shadow3 + shadow4 + shadow5 + shadow6 + shadow7 + shadow8) * (1.0 / 9.0);
}

float GetShadowMediumStar(float4 projection, float texelSize)
{
	float4 offset = float4(texelSize, -texelSize, 0.0, 0.0) * projection.w;
	
    float shadow0 = SampleShadow(shadowMap, shadowMapsampler, projection + offset.xxzz).x;
    float shadow1 = SampleShadow(shadowMap, shadowMapsampler, projection + offset.xyzz).x;
    float shadow2 = SampleShadow(shadowMap, shadowMapsampler, projection + offset.yxzz).x;
    float shadow3 = SampleShadow(shadowMap, shadowMapsampler, projection + offset.yyzz).x;
    float shadow4 = SampleShadow(shadowMap, shadowMapsampler, projection).x;

    return (shadow0 + shadow1 + shadow2 + shadow3) * (1.0 / 5.0);
}

float GetShadowLow(float4 projection)
{
    return SampleShadow(shadowMap, shadowMapsampler, projection).x;
}

#endif