Material resource
{
	MaterialType _type = Opaque;
	VideoProgram _passes
	[ 
		"normal"
		"depth"
		"reflect"
	]
}

VideoProgram normal
{
	VertexLayout _vertexLayout = "MeshLayout\TerrainLayout.rsc";
	VertexProgram _vertexProgram = "vs";
	PixelProgram _pixelProgram = "ps";
	RenderState _renderState = "Material\Opaque\OpaqueMaterial.rsc:rs";
}

VideoProgram depth
{
	VertexLayout _vertexLayout = "MeshLayout\TerrainDepthLayout.rsc";
	VertexProgram _vertexProgram = "vsDepth";
	PixelProgram _pixelProgram = "psDepth";
	RenderState _renderState = "Material\Opaque\OpaqueMaterial.rsc:rsDepth";
}	

VideoProgram reflect
{
	VertexLayout _vertexLayout = "MeshLayout\TerrainLayout.rsc";
	VertexProgram _vertexProgram = "vs";
	PixelProgram _pixelProgram = "psRef";
	RenderState _renderState = "Material\Opaque\OpaqueMaterial.rsc:rsRef";
}

VertexProgram vs
{
	VertexLayout _input = "MeshLayout/TerrainLayout.rsc";
	String _includes
	[
		"Material/Instancing.hlsl"
		"Material/Lighting.hlsl"
		"Material/UnderWater.hlsl"
	]

	Output _outputs 
	[
		{ String _type = "float4"; String _name = "position";		Usage _usage = Position;			int _index = 0; }
		{ String _type = "float4"; String _name = "texcoord";		Usage _usage = TextureCoordinate;	int _index = 0; }
		{ String _type = "float3"; String _name = "lightFog";		Usage _usage = TextureCoordinate;	int _index = 1; }
		{ String _type = "float3"; String _name = "worldNormal";	Usage _usage = TextureCoordinate;	int _index = 2; }
		{ String _type = "float2"; String _name = "heightWater";	Usage _usage = TextureCoordinate;	int _index = 3; }
	]

	String _functions
	[
		"
			void Common(float4x4 localToWorld, float4 position, float3 normal, float2 texcoord, 
				out float3 worldPosition, out float4 outputPosition, out float4 outputTexcoord, 
				out float3 worldNormal, out float2 heightWater,	out float3 lightFog)
			{
				position.z /= 256.0;
				worldPosition = mul((float3x4)localToWorld, float4(position.xyz, 1.0));
				outputPosition = mul(gc_worldToProjection, float4(worldPosition, 1.0));
	
				normal = normalize(normal * 2 - 1);
	
				lightFog = GetLightingFog(normal, outputPosition.z);
	
				outputTexcoord.xy = position.xy / 16.0;
				outputTexcoord.zw = texcoord.xy / 16384.0;
	
				worldNormal.xyz = normal;
				heightWater.x = position.z / 16.0;
				heightWater.y = GetUnderWaterAmount(position.z);
			}
		"
	]

	Variant _variants
	[
		{
			String _text = "
				float4x4 localToWorld = GetLocalToWorld(0);
				float3 worldPosition;
				Common(localToWorld, input.position, input.normal.xyz, input.texcoord, worldPosition, output.position, output.texcoord, output.worldNormal, 
					output.heightWater, output.lightFog);
			";
		}
		{
			Output _outputs 
			[
				{ String _type = "float4"; String _name = "shadowProjection";	Usage _usage = TextureCoordinate;	int _index = 4; }
			]
			String _text = "
				float4x4 localToWorld = GetLocalToWorld(0);
				float3 worldPosition;
				Common(localToWorld, input.position, input.normal.xyz, input.texcoord, worldPosition, output.position, output.texcoord, output.worldNormal, 
					output.heightWater, output.lightFog);
				output.shadowProjection = mul(lc_shadowProjection0, float4(worldPosition, 1.0f));
			";
		}
		{
			Output _outputs 
			[
				{ String _type = "float4"; String _name = "shadowProjection";	Usage _usage = TextureCoordinate;	int _index = 4; }
			]
			String _text = "
				float4x4 localToWorld = GetLocalToWorld(0);
				float3 worldPosition;
				Common(localToWorld, input.position, input.normal.xyz, input.texcoord, worldPosition, output.position, output.texcoord, output.worldNormal, 
					output.heightWater, output.lightFog);
				output.shadowProjection = mul(lc_shadowProjection0, float4(worldPosition, 1.0f));
			";
		}
		{
			Output _outputs 
			[
				{ String _type = "float4"; String _name = "shadowProjection";	Usage _usage = TextureCoordinate;	int _index = 4; }
			]
			String _text = "
				float4x4 localToWorld = GetLocalToWorld(0);
				float3 worldPosition;
				Common(localToWorld, input.position, input.normal.xyz, input.texcoord, worldPosition, output.position, output.texcoord, output.worldNormal, 
					output.heightWater, output.lightFog);
				output.shadowProjection = mul(lc_shadowProjection0, float4(worldPosition, 1.0f));
			";
		}
	]
}

PixelProgram ps
{
	VertexProgram _input = "vs";
	String _includes
	[
		"Material/Texture.hlsl"
		"Material/Shadows.hlsl"
	]

	Texture _textureList
	[
		{ bool _2d = true;	String _name = "grass";	int _bindIndex = 0;	bool _exposed = true; }
		{ bool _2d = true;	String _name = "dirt";	int _bindIndex = 1;	bool _exposed = true; }
		{ bool _2d = true;	String _name = "snow";	int _bindIndex = 2;	bool _exposed = true; }
		{ bool _2d = true;	String _name = "rock";	int _bindIndex = 3;	bool _exposed = true; }
		{ bool _2d = true;	String _name = "shadowMap";	int _bindIndex = 5;	bool _exposed = false; bool _shadowMap = true; }
		{ bool _2d = true;	String _name = "transitionMap";	int _bindIndex = 6;	bool _exposed = false; }
	]

	String _functions
	[
		"
			float4 GetColor(float4 texcoord, float3 lightFog, float3 worldNormal, float2 heightWater, float shadow)
			{
				float4 grassmap = Sample2D(grass, grasssampler, texcoord.xy);
				float4 dirtmap = Sample2D(dirt, dirtsampler, texcoord.xy);
				float4 snowmap = Sample2D(snow, snowsampler, texcoord.xy);
				float4 transition = Sample2D(transitionMap, transitionMapsampler, texcoord.zw);
    
				float4 rocka = Sample2D(rock, rocksampler, float2(texcoord.y, heightWater.x));
				float4 rockb = Sample2D(rock, rocksampler, float2(texcoord.x, heightWater.x));
				float4 rockc = Sample2D(rock, rocksampler, texcoord.xy);
				float3 n = abs(worldNormal.xyz);
				float4 rock = rocka * n.x + rockb * n.y + rockc * n.z;    
		    
				float3 diffuse = grassmap.xyz;
				diffuse = lerp(diffuse, dirtmap.xyz, transition.b);
				diffuse = lerp(diffuse, rock.xyz, transition.g);
    
				float snowAmount = worldNormal.z * worldNormal.z;
				snowAmount *= snowAmount * pc_userPosition.x;
				snowAmount = max(snowAmount, transition.w);
				diffuse = lerp(diffuse, snowmap.xyz, snowAmount);
    
				float4 output;
				diffuse.xyz = lerp(diffuse.xyz, float3(0.025, 0.123, 0.19), saturate(heightWater.y));
				output.xyz = ComputeLighting(lightFog, diffuse, shadow, float3(1, 1, 1), transition.rrr);
				
				output.w = heightWater.y;

				return output;
			}
		"
	]

	Variant _variants
	[
		{
			String _text = "output.color = GetColor(input.texcoord, input.lightFog, input.worldNormal, input.heightWater, 1);";
		}
		{
			String _text = "output.color = GetColor(input.texcoord, input.lightFog, input.worldNormal, input.heightWater, GetShadowLow(input.shadowProjection));";
		}
		{
			String _text = "output.color = GetColor(input.texcoord, input.lightFog, input.worldNormal, input.heightWater, GetShadowMedium(input.shadowProjection, pc_texelSize.x));";
		}
		{
			String _text = "output.color = GetColor(input.texcoord, input.lightFog, input.worldNormal, input.heightWater, GetShadowHigh(input.shadowProjection, pc_texelSize.x));";
		}
	]
}

PixelProgram psRef : "ps"
{
	Variant _variants
	[
		{
			String _text = "output.color = GetColor(input.texcoord, input.lightFog, input.worldNormal, input.heightWater, 1); clip(-(input.heightWater.y));";
		}
		{
			String _text = "output.color = GetColor(input.texcoord, input.lightFog, input.worldNormal, input.heightWater, GetShadowLow(input.shadowProjection)); clip(-(input.heightWater.y));";
		}
		{
			String _text = "output.color = GetColor(input.texcoord, input.lightFog, input.worldNormal, input.heightWater, GetShadowMedium(input.shadowProjection, pc_texelSize.x)); clip(-(input.heightWater.y));";
		}
		{
			String _text = "output.color = GetColor(input.texcoord, input.lightFog, input.worldNormal, input.heightWater, GetShadowHigh(input.shadowProjection, pc_texelSize.x)); clip(-(input.heightWater.y));";
		}
	]
}

VertexProgram vsDepth
{
	VertexLayout _input = "MeshLayout/TerrainDepthLayout.rsc";
	String _includes
	[
		"Material/Instancing.hlsl"
	]

	Output _outputs 
	[
		{ String _type = "float4"; String _name = "position";		Usage _usage = Position;			int _index = 0; }
	]

	Variant _variants
	[
		{
			String _text = "
				float4x4 localToWorld = GetLocalToWorld(0);
				float4 position = input.position;

				position.z *= 1.0 / 256.0;
				float3 worldPosition = mul((float3x4)localToWorld, float4(position.xyz, 1.0));
				output.position = mul(gc_worldToProjection, float4(worldPosition, 1.0));
			";
		}
	]
}

PixelProgram psDepth
{
	VertexProgram _input = "vsDepth";

	Variant _variants
	[
		{
			String _text = "
				output.color = float4(1.0, 1.0, 1.0, 1.0);
			";
		}
	]
}